@@ -143,11 +143,12 @@ impl Allocator {
 
         // This may cause disk leaks in case of crash, but
         // keep the system safe
-        for page in freelist.list {
+        for pos in 0..freelist.list.len() {
+            let page = freelist.list[pos];
             if page != 0 {
                 let lp = dr.load_page(page)?;
                 if !lp.is_free()? {
-                    freelist.list[lp.get_size_exp() as usize] = 0;
+                    freelist.list[pos] = 0;
                 }
             }
         }
