@@ -943,6 +943,12 @@ impl PersyImpl {
         }
     }
 }
+impl Drop for PersyImpl {
+    fn drop(&mut self) {
+        self.allocator.flush_free_list().unwrap();
+        self.allocator.disc_sync().unwrap();
+    }
+}
 
 pub fn exp_from_content_size(size: u64) -> u8 {
     // content + size + match_pointer:page+ match_pointer:pos  + page_header
