@@ -117,9 +117,14 @@ pub struct InternalSnapshots {
 }
 impl InternalSnapshots {
     fn next_snapshot_id(&mut self) -> SnapshotId {
-        //TODO: this may overflow and should start again from 0 double check overflow failures
         let snapshot_id = self.snapshot_sequence;
-        self.snapshot_sequence += 1;
+        // This should be the default behaviour of the u64 anyway, let's hope the compiler trows
+        // this away
+        if self.snapshot_sequence == u64::MAX {
+            self.snapshot_sequence = 0;
+        } else {
+            self.snapshot_sequence += 1;
+        }
         snapshot_id
     }
     fn search(&self, snapshot_id: SnapshotId) -> Result<usize, usize> {
@@ -143,6 +148,59 @@ impl InternalSnapshots {
             panic!("try to acquire a not existing snapshot")
         }
     }
+
+    fn clear_from(&mut self, snapshot_id: SnapshotId) -> PRes<(Vec<FreedPage>, Vec<JournalId>)> {
+        let snapshot_sequence = self.snapshot_sequence;
+        let mut free_pages = Vec::new();
+        let mut journal_ids = Vec::new();
+        if let Ok(index) = self.search(snapshot_id) {
+            let size = index + 1;
+            let mut old = Vec::with_capacity(size);
+            old.extend_from_slice(&self.active_snapshots[0..size]);
+            self.active_snapshots = self.active_snapshots.split_off(size);
+            for tx in old {
+                if let Some(id) = tx.journal_id.clone() {
+                    journal_ids.push(id.clone());
+                }
+                if let Some(entries) = tx.entries {
+                    for record in entries {
+                        if let Some(v) = self.mapping.get_mut(&record.id) {
+                            if let Ok(index) =
+                                v.binary_search_by(|n| search(n.snapshot_id, snapshot_id, snapshot_sequence))
+                            {
+                                v.truncate(index);
+                            }
+                        }
+                    }
+                }
+                if let Some(pages) = tx.freed_pages {
+                    free_pages.extend(pages);
+                }
+            }
+        }
+        Ok((free_pages, journal_ids))
+    }
+
+    fn release(&mut self, snapshot_id: SnapshotId) -> PRes<(Vec<FreedPage>, Vec<JournalId>)> {
+        let mut clear_id = None;
+        if let Ok(index) = self.search(snapshot_id) {
+            let mut loop_index = index;
+            while let Some(snap) = self.active_snapshots.get_mut(loop_index) {
+                snap.reference_count -= 1;
+                if snap.reference_count > 0 {
+                    break;
+                }
+                clear_id = Some(snap.snapshot_id);
+                loop_index += 1;
+            }
+        }
+
+        if let Some(c_id) = clear_id {
+            self.clear_from(c_id)
+        } else {
+            Ok((Vec::new(), Vec::new()))
+        }
+    }
 }
 
 pub struct Snapshots {
@@ -368,62 +426,14 @@ impl Snapshots {
         })
     }
 
+    #[allow(unused)]
     fn clear_from(&self, snapshot_id: SnapshotId) -> PRes<(Vec<FreedPage>, Vec<JournalId>)> {
-        let mut lock = self.lock.lock()?;
-        let snapshot_sequence = lock.snapshot_sequence;
-        let mut free_pages = Vec::new();
-        let mut journal_ids = Vec::new();
-        if let Ok(index) = lock.search(snapshot_id) {
-            let size = index + 1;
-            let mut old = Vec::with_capacity(size);
-            old.extend_from_slice(&lock.active_snapshots[0..size]);
-            lock.active_snapshots = lock.active_snapshots.split_off(size);
-            for tx in old {
-                if let Some(id) = tx.journal_id.clone() {
-                    journal_ids.push(id.clone());
-                }
-                if let Some(entries) = tx.entries {
-                    for record in entries {
-                        if let Some(v) = lock.mapping.get_mut(&record.id) {
-                            if let Ok(index) =
-                                v.binary_search_by(|n| search(n.snapshot_id, snapshot_id, snapshot_sequence))
-                            {
-                                v.truncate(index);
-                            }
-                        }
-                    }
-                }
-                if let Some(pages) = tx.freed_pages {
-                    free_pages.extend(pages);
-                }
-            }
-        }
-        Ok((free_pages, journal_ids))
+        self.lock.lock()?.clear_from(snapshot_id)
     }
 
     fn release(&self, snapshot_id: SnapshotId) -> PRes<(Vec<FreedPage>, Vec<JournalId>)> {
-        //TODO: This work fine but can cause problems if double release is called for the same id,
-        //to refactor to something a bit more safe
-        let mut clear_id = None;
-        {
-            let mut lock = self.lock.lock()?;
-            if let Ok(index) = lock.search(snapshot_id) {
-                let mut loop_index = index;
-                while let Some(snap) = lock.active_snapshots.get_mut(loop_index) {
-                    snap.reference_count -= 1;
-                    if snap.reference_count > 0 {
-                        break;
-                    }
-                    clear_id = Some(snap.snapshot_id);
-                    loop_index += 1;
-                }
-            }
-        }
-        if let Some(c_id) = clear_id {
-            self.clear_from(c_id)
-        } else {
-            Ok((Vec::new(), Vec::new()))
-        }
+        let mut lock = self.lock.lock()?;
+        lock.release(snapshot_id)
     }
 }
 
